#include "configdialog2.h"
#include "ui_configdialog2.h"

#include <QStandardItem>
#include <QStandardItemModel>
#include <QIcon>

#include "bookmoocherdata.h"
#include "defines.h"
#include <QSettings>

ConfigDialog2::ConfigDialog2(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ConfigDialog2)
{
    ui->setupUi(this);

    // Selettore delle operazioni
    QStandardItemModel *model = new QStandardItemModel(this);
    populateActionsIconViewModel(model);
    ui->settingsIconView->setModel(model);
    connect(ui->settingsIconView, SIGNAL(clicked(const QModelIndex&)), this, SLOT(settingsIconViewClicked(const QModelIndex&)));

    // Sistema la spin box
	ui->timeIntervalSpinBox->setRange(CFG_MIN_FETCH_INTERVAL, CFG_MAX_FETCH_INTERVAL);

	// Aggiunge i model
	BookMoocherData::populateCountriesCombo(ui->countriesComboBox);
	ui->countriesListView->setModel(new QStandardItemModel());

	// Popola dai settaggi
    QSettings settings;

	// Intervallo di fetch
	// Verifica che il settaggio sia valido
	bool ok;
	if (settings.value(CFG_BOOK_INFO_FETCH_INTERVAL).canConvert(QVariant::Int)) {
		int time_interval = settings.value(CFG_BOOK_INFO_FETCH_INTERVAL).toInt(&ok);

		if (ok) {
			if (time_interval >= ui->timeIntervalSpinBox->minimum() && time_interval <= ui->timeIntervalSpinBox->maximum())
				ui->timeIntervalSpinBox->setValue(time_interval);
			else
				ui->timeIntervalSpinBox->setValue( CFG_DEFAULT_FETCH_INTERVAL );
		} else {
			ui->timeIntervalSpinBox->setValue( CFG_DEFAULT_FETCH_INTERVAL );
		}
	} else {
		ui->timeIntervalSpinBox->setValue( CFG_DEFAULT_FETCH_INTERVAL );
	}
	// Nazioni
	// Verifica la correttezza del settaggio
	if (settings.value(CFG_COUNTRIES_LIST, QVariant(QVariant::StringList)).canConvert(QVariant::StringList)) {
		QStringListIterator iter( settings.value(CFG_COUNTRIES_LIST).toStringList() );
		QStandardItem *item;
		while (iter.hasNext()) {
			QString s = iter.next();
			item = new QStandardItem( BookMoocherData::getCountries().value( s ) );
			item->setData( s );
			item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			dynamic_cast<QStandardItemModel*>(ui->countriesListView->model())->appendRow(item);
		}
		dynamic_cast<QStandardItemModel*>(ui->countriesListView->model())->sort(0);
	}

	// Età massima dei libri
	ui->bookAgeSpinBox->setRange(CFG_BOOK_LIST_MAX_BOOK_AGE_MIN, CFG_BOOK_LIST_MAX_BOOK_AGE_MAX);
	ui->bookAgeSpinBox->setValue(CFG_DEFAULT_MAX_BOOK_AGE);
	if (settings.contains(CFG_BOOK_LIST_MAX_BOOK_AGE)) {
		int age = settings.value(CFG_BOOK_LIST_MAX_BOOK_AGE).toInt(&ok);
		if (ok) {
			if (age >= CFG_BOOK_LIST_MAX_BOOK_AGE_MIN && age <= CFG_BOOK_LIST_MAX_BOOK_AGE_MAX) {
				ui->bookAgeSpinBox->setValue(age);
			}
		}
	}

	// Numero massimo di libri da mostrare
	ui->bookListLengthSpinBox->setRange(CFG_BOOK_LIST_MIN_ITEMS_COUNT, CFG_BOOK_LIST_MAX_ITEMS_COUNT);
	ui->bookListLengthSpinBox->setValue(CFG_DEFAULT_MAX_BOOK_LIST_ITEMS);

	if (settings.contains(CFG_BOOK_LIST_MAX_ITEMS)) {
		int cnt = settings.value(CFG_BOOK_LIST_MAX_ITEMS).toInt(&ok);
		if (ok) {
			if (cnt >= CFG_BOOK_LIST_MIN_ITEMS_COUNT && cnt <= CFG_BOOK_LIST_MAX_ITEMS_COUNT) {
				ui->bookListLengthSpinBox->setValue(cnt);
			}
		}
	}

	// Azione per il doppio click
	if (settings.contains(CFG_BOOK_LISTVIEW_DOUBLECLICK)) {
		int bldca = settings.value(CFG_BOOK_LISTVIEW_DOUBLECLICK).toInt(&ok);
		if (ok) {
			switch(bldca) {
			case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_MOOCH:
				ui->bookListDblClkActionCombo->setCurrentIndex(0);
				break;
			case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SHOW_BOOK_INFO:
				ui->bookListDblClkActionCombo->setCurrentIndex(1);
				break;
			case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_ADD_TO_WISHLIST:
				ui->bookListDblClkActionCombo->setCurrentIndex(2);
			break;
			case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SAVE_FOR_LATER:
				ui->bookListDblClkActionCombo->setCurrentIndex(3);
			break;
			case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SEARCH_ANOBII:
				ui->bookListDblClkActionCombo->setCurrentIndex(4);
			break;
			case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SEARCH_ANOBII_BY_ISBN:
				ui->bookListDblClkActionCombo->setCurrentIndex(5);
			break;
			default:
				ui->bookListDblClkActionCombo->setCurrentIndex(0);
			}
		}
	}

	// Ricerca su anobii
	int a_sm = settings.value(CFG_ANOBII_SEARCH_MODE).toInt();
	if (a_sm == CFG_ANOBII_SEARCH_BY_ISBN) {
		ui->anobiiSearchModeComboBox->setCurrentIndex(1);
	} else {
		ui->anobiiSearchModeComboBox->setCurrentIndex(0);
	}

	// Seleziona le giuste pagine di configurazione
	// ui->settingsIconView->setCurrentIndex( ui->settingsIconView->model()->index(0, 0) );
	ui->configStackedWidget->setCurrentWidget(ui->pageCountrySettings);
}

void ConfigDialog2::populateActionsIconViewModel(QStandardItemModel *model) {
	// Selettore delle nazioni
	QStandardItem *item = new QStandardItem(QIcon(":/dummy_icon.png"), tr("Country"));
	item->setData("country");
	model->appendRow(item);

	// Connessione
	item = new QStandardItem(QIcon(":/dummy_icon.png"), tr("Connection"));
	item->setData("connection");
	model->appendRow(item);

	// Book list
	item = new QStandardItem(QIcon(":/dummy_icon.png"), tr("Book list"));
	item->setData("booklist");
	model->appendRow(item);

	// Anobii
	item = new QStandardItem(QIcon(":/dummy_icon.png"), tr("aNobii.com integration"));
	item->setData("anobii");
	model->appendRow(item);

}

void ConfigDialog2::settingsIconViewClicked(const QModelIndex &index) {
	// Estrae i dati associati e passa alla tab corrispondente
	if (index.isValid()) {
		QStandardItem *i = dynamic_cast<QStandardItemModel*>(ui->settingsIconView->model())->itemFromIndex(index);
		if (i->data().toString() == "country") {
			ui->configStackedWidget->setCurrentWidget( ui->pageCountrySettings);
		} else if (i->data().toString() == "connection") {
			ui->configStackedWidget->setCurrentWidget( ui->pageConnectionSettings);
		} else if (i->data().toString() == "booklist") {
			ui->configStackedWidget->setCurrentWidget( ui->pageBookListSettings);
		} else if (i->data().toString() == "anobii") {
			ui->configStackedWidget->setCurrentWidget( ui->pageAnobiiSettings);
		}
	}
}

ConfigDialog2::~ConfigDialog2()
{
    delete ui;
}

int ConfigDialog2::getFeedInterval() const {
    return ui->timeIntervalSpinBox->value();
}

void ConfigDialog2::on_addCountryButton_clicked()
{
	// aggiunge se non è già presente l'elemento attualmente selezionato
	// nella combobox alla listview

	QVariant value( ui->countriesComboBox->itemData( ui->countriesComboBox->currentIndex() ) );

	bool found_item = false;
	QStandardItem *item;
	for (int i = 0; i < ui->countriesListView->model()->rowCount(); i++ ) {
		item = dynamic_cast<QStandardItemModel*>(ui->countriesListView->model())->item(i);
		if (item != NULL) {
			if (item->data() == value) {
				found_item = true;
			}
		}
	}

	if ( !found_item ) {
		item = new QStandardItem( ui->countriesComboBox->itemText(ui->countriesComboBox->currentIndex() ) );
		item->setData( ui->countriesComboBox->itemData( ui->countriesComboBox->currentIndex() ) );
		item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		dynamic_cast<QStandardItemModel*>(ui->countriesListView->model())->appendRow(item);
		dynamic_cast<QStandardItemModel*>(ui->countriesListView->model())->sort(0);
	}
}

void ConfigDialog2::on_delCountryButton_clicked()
{
	ui->countriesListView->model()->removeRows(ui->countriesListView->currentIndex().row(), 1);
}


/*
	Salva i settaggi impostati in questa form
*/
void ConfigDialog2::saveSettings() const {
	// Salva i settaggi
	QSettings settings;

	// Semplici stringhe
	settings.setValue(CFG_BOOK_INFO_FETCH_INTERVAL, QVariant( ui->timeIntervalSpinBox->value() ));

	// Liste di stringhe
	QStringList sl;
	for (int i = 0; i < ui->countriesListView->model()->rowCount(); i++) {
		sl << dynamic_cast<QStandardItemModel*>(ui->countriesListView->model())->item(i)->data().toString() ;
	}
	settings.setValue(CFG_COUNTRIES_LIST, QVariant(sl));

	// Età dei libri
	settings.setValue(CFG_BOOK_LIST_MAX_BOOK_AGE, QVariant( ui->bookAgeSpinBox->value() ));

	// Numero massimo di elementi
	settings.setValue(CFG_BOOK_LIST_MAX_ITEMS, QVariant( ui->bookListLengthSpinBox->value() ));

	// Azione col doppio click sulla lista dei libri
	switch(ui->bookListDblClkActionCombo->currentIndex()) {
		case 0: settings.setValue(CFG_BOOK_LISTVIEW_DOUBLECLICK, QVariant(CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_MOOCH)); break;
		case 1: settings.setValue(CFG_BOOK_LISTVIEW_DOUBLECLICK, QVariant(CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SHOW_BOOK_INFO)); break;
		case 2: settings.setValue(CFG_BOOK_LISTVIEW_DOUBLECLICK, QVariant(CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_ADD_TO_WISHLIST)); break;
		case 3: settings.setValue(CFG_BOOK_LISTVIEW_DOUBLECLICK, QVariant(CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SAVE_FOR_LATER)); break;

		case 4: settings.setValue(CFG_BOOK_LISTVIEW_DOUBLECLICK, QVariant(CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SEARCH_ANOBII)); break;
		case 5: settings.setValue(CFG_BOOK_LISTVIEW_DOUBLECLICK, QVariant(CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SEARCH_ANOBII_BY_ISBN)); break;
		default:
			settings.setValue(CFG_BOOK_LISTVIEW_DOUBLECLICK, QVariant(CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_MOOCH));
	}

	// Ricerca su anobii
	if (ui->anobiiSearchModeComboBox->currentIndex() == 1) {
		settings.setValue(CFG_ANOBII_SEARCH_MODE, QVariant(CFG_ANOBII_SEARCH_BY_ISBN));
	} else {
		settings.setValue(CFG_ANOBII_SEARCH_MODE, QVariant(CFG_ANOBII_SEARCH_BY_TITLE));
	}
}

