#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "defines.h"

#include <QSettings>
#include <QVariant>

#include <QMessageBox>
#include <QStringList>

#include <QDesktopServices>
#include <QDir>
#include <QFile>
#include <QTextStream>
#include <QLocale>
#include <QDateTime>
#include <QTimer>
#include <QUrl>
#include <QLocale>

#include <QSystemTrayIcon>
#include <QMenu>

#include <QNetworkReply>

#include <QDebug>

#include "booklistmodel.h"
#include "booklistitemdelegate.h"
#include "aboutdialog.h"
#include "BMException.h"
#include "configdialog2.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
	ui(new Ui::MainWindow), is_updating(false)
{
    ui->setupUi(this);

	// Connette il segnale di uscita dall'applicazione
	// connect (ui->actionEsci, SIGNAL(triggered()), this, SLOT(close()));

	// connette il segnale per aggiornare da menu
	connect(ui->actionUpdate, SIGNAL(triggered()), this, SLOT(checkForNewBooks()));

	// Prepara la pulsantiera
	main_button_group = new QButtonGroup(this);
	main_button_group->setExclusive(true);
	main_button_group->addButton(ui->showBookListToolButton, 0);
	main_button_group->addButton(ui->showBookInfoToolButton, 1);
	main_button_group->addButton(ui->notesToolButton, 2);
	connect(main_button_group, SIGNAL(buttonClicked(int)), this, SLOT(selectStackedWidget(int)) );
	ui->showBookListToolButton->click();

	// Verifica se ci siano dei settaggi, nel caso non
	// ci siano mette quelli di default
	QSettings settings;

	// Copia se esiste il vecchio parametro di configurazione
	if (settings.contains(CFG_OLD_RSS_FETCH_INTERVAL)) {
		settings.setValue(CFG_BOOK_INFO_FETCH_INTERVAL, settings.value(CFG_OLD_RSS_FETCH_INTERVAL));
		settings.remove(CFG_OLD_RSS_FETCH_INTERVAL);
	}

	if (settings.contains(CFG_BOOK_INFO_FETCH_INTERVAL)) {
		// Controlli di sanità sul dato
		bool ok;
		int i = settings.value(CFG_BOOK_INFO_FETCH_INTERVAL).toInt(&ok);
		if (ok) {
			if (i < CFG_MIN_FETCH_INTERVAL || i > CFG_MAX_FETCH_INTERVAL) settings.setValue(CFG_BOOK_INFO_FETCH_INTERVAL, CFG_DEFAULT_FETCH_INTERVAL);
		} else {
			settings.setValue(CFG_BOOK_INFO_FETCH_INTERVAL, CFG_DEFAULT_FETCH_INTERVAL);
		}
	} else {
		settings.setValue(CFG_BOOK_INFO_FETCH_INTERVAL, CFG_DEFAULT_FETCH_INTERVAL);
	}
	if (!settings.contains(CFG_BOOK_LIST_MAX_ITEMS)) settings.setValue(CFG_BOOK_LIST_MAX_ITEMS, CFG_DEFAULT_MAX_BOOK_LIST_ITEMS);
	if (!settings.contains(CFG_BOOK_LIST_MAX_BOOK_AGE)) settings.setValue(CFG_BOOK_LIST_MAX_BOOK_AGE, CFG_DEFAULT_MAX_BOOK_AGE);

	// Modalità di ricerca su anobii
	anobii_search_mode = CFG_DEFAULT_ANOBII_SEARCH_MODE;
	if (settings.contains(CFG_ANOBII_SEARCH_MODE)) {
		int sm = settings.value(CFG_ANOBII_SEARCH_MODE).toInt();
		switch (sm) {
		case CFG_ANOBII_SEARCH_BY_TITLE:
		case CFG_ANOBII_SEARCH_BY_ISBN:
			anobii_search_mode = sm;
			break;
		default:
			settings.setValue(CFG_ANOBII_SEARCH_MODE, QVariant(CFG_DEFAULT_ANOBII_SEARCH_MODE));
		}
	} else {
		settings.setValue(CFG_ANOBII_SEARCH_MODE, QVariant(CFG_DEFAULT_ANOBII_SEARCH_MODE));
	}

	// Inizializza il core, se c'è un errore esce dall'applicazione
	try {
		bookmoochercore = new BookMoocherCore(this);
	} catch (BMException e) {
		QMessageBox::critical(this, tr("Warning!"), e.message());
		qApp->quit();
	}

	// Connette i segnali di aggiornamento al core
	connect(bookmoochercore, SIGNAL(finished(int, BookList&, BookMoocherCore::BookMoocherUpdateStatus, const QString&)), this, SLOT(finishedBookFetch(int, BookList&, BookMoocherCore::BookMoocherUpdateStatus, const QString&)) );
	connect(bookmoochercore, SIGNAL(finishedBookInfoFetch(BookInfoItem &, BookMoocherCore::BookMoocherUpdateStatus, const QString&)), this, SLOT(finishedBookInfoFetch(BookInfoItem &, BookMoocherCore::BookMoocherUpdateStatus, const QString& ) ));


	// Inserisce il model giusto nel view
	this->model = new BookListModel();
	this->proxy_model = new BookListProxyModel(this);
	this->proxy_model->setFilterCaseSensitivity(Qt::CaseInsensitive);
	this->proxy_model->setSourceModel(this->model);
	connect(ui->filterLineEdit, SIGNAL(textEdited(const QString&)), this->proxy_model, SLOT(setFilterWildcard(const QString&)));

	ui->listView->setModel( proxy_model );

	ui->listView->setItemDelegate( new BookListItemDelegate(this) );

	// Azione al doppio click
	book_list_view_action = CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_MOOCH;
	if (settings.contains(CFG_BOOK_LISTVIEW_DOUBLECLICK)) {
		if (settings.value(CFG_BOOK_LISTVIEW_DOUBLECLICK).canConvert(QVariant::Int)) {
			int blva = settings.value(CFG_BOOK_LISTVIEW_DOUBLECLICK).toInt();
			switch (blva) {
			case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_MOOCH:
			case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SHOW_BOOK_INFO:
			case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_ADD_TO_WISHLIST:
			case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SAVE_FOR_LATER:
				book_list_view_action = blva;
				break;
			}
		} else {
			settings.setValue(CFG_BOOK_LISTVIEW_DOUBLECLICK, QVariant(CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_MOOCH));
		}
	} else {
		// Di default esegue il mooch
		settings.setValue(CFG_BOOK_LISTVIEW_DOUBLECLICK, QVariant(CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_MOOCH));
	}

	// Controlla la versione di bookmoocher e nel caso aggiorna la lista dei libri
	QString error_msg;
	if (settings.contains(CFG_APP_VERSION)) {
		if (settings.value(CFG_APP_VERSION).toString() == APP_VERSION) {
			bookmoochercore->loadBookList(model);
		} else {
			// Rimuove i generi
			settings.remove(CFG_GENRES_LIST);
			// Aggiorna la lista
			if (bookmoochercore->convertBookListToLatestVersion(model, error_msg))
				settings.setValue(CFG_APP_VERSION, QVariant(APP_VERSION));
		}
	} else {
		// Rimuove i generi
		settings.remove(CFG_GENRES_LIST);
		// Una delle prime versioni, deve convertire
		if (bookmoochercore->convertBookListToLatestVersion(model, error_msg))
			settings.setValue(CFG_APP_VERSION, QVariant(APP_VERSION));
	}
	if (!error_msg.isEmpty()) {
		QMessageBox::warning(this, tr("Warning!"), error_msg);
	}

	// Crea l'icona nella systemtray
	setupSystemTrayIcon();
	connect(systray_icon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(systrayIconActivated(QSystemTrayIcon::ActivationReason)));
	systray_icon->show();

	// Carica le annotazioni
	loadNotes();

	// Avvia il timer
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(checkForNewBooks()));
	timer->setInterval(60000 * settings.value(CFG_BOOK_INFO_FETCH_INTERVAL).toInt());
	timer->start();

	// Esegue se necessario l'aggiornamento dei dati
	if (bookmoochercore->getLastUpdateTime().isValid()) {
		// Mette l'opportuno tooltip
		systray_icon->setToolTip(
				tr("Last update at %1").arg(QLocale::system().toString(bookmoochercore->getLastUpdateTime(), QLocale::ShortFormat))
		);

		// Se la data dell'ultimo fetch è troppo in là nel tempo
		qDebug() << "LAST UPD:" << bookmoochercore->getLastUpdateTime().secsTo( QDateTime::currentDateTime() );
		if (bookmoochercore->getLastUpdateTime().secsTo( QDateTime::currentDateTime() ) > (settings.value(CFG_BOOK_INFO_FETCH_INTERVAL).toInt() * 60 )) {
			checkForNewBooks();
		}
	} else {
		checkForNewBooks();
	}
}

void MainWindow::enableButtonGroup(bool enable) {
	QListIterator<QAbstractButton*> iter(main_button_group->buttons());

	while (iter.hasNext()) {
		QToolButton *btn = dynamic_cast<QToolButton*>(iter.next());
		btn->setEnabled(enable);
	}
}

void MainWindow::selectStackedWidget(int id) {
	if (id == main_button_group->id(ui->showBookInfoToolButton)) {
		// Se c'è un elemento selezionato nella lista dei libri
		if (ui->listView->currentIndex().isValid()) {
			BookListItem sel_item = ui->listView->currentIndex().data().value<BookListItem>();
			if (sel_item.ASIN != book_info_shown.ASIN) {
				// Esegue l'aggiornamento
				ui->bookInfoTextBrowser->setText(tr("Updating..."));
				ui->bookInfoTextBrowser->setEnabled(false);
				bookmoochercore->fetchBookInfo(sel_item);

			}
		}
	}
	ui->stackedWidget->setCurrentIndex(id);
}

void MainWindow::finishedBookInfoFetch(BookInfoItem &bookinfo, BookMoocherCore::BookMoocherUpdateStatus status, const QString& error_msg ) {
	if (status == BookMoocherCore::UpdateOK) {
		QString str = "<html>\n<head>\n<title>";
		str += bookinfo.title;
		str += "</title>\n</head>\n<body>\n";
		str += "<h2>" + bookinfo.title + "</h2>\n";
		str += "<table>\n";
		str += "<tr><td><strong>" + tr("Author:") + "</strong></td><td>" + bookinfo.author + "</td></tr>\n";
		str += "<tr><td><strong>" + tr("Publisher:") + "</strong></td><td>" + bookinfo.publisher + "</td></tr>\n";
		str += "<tr><td><strong>" + tr("Published:") + "</strong></td><td>" + QLocale::system().toString(bookinfo.publication_date, QLocale::ShortFormat) + "</td></tr>\n";
		str += "<tr><td><strong>" + tr("ISBN:") + "</strong></td><td>" + bookinfo.ISBN + "</td></tr>\n";
		str += "<tr><td><strong>" + tr("Edition:") + "</strong></td><td>" + bookinfo.edition + "</td></tr>\n";
		str += "<tr><td><strong>" + tr("Binding:") + "</strong></td><td>" + bookinfo.binding + "</td></tr>\n";
		str += "<tr><td><strong>" + tr("Number of pages:") + "</strong></td><td>" + QString::number(bookinfo.number_of_pages) + "</td></tr>\n";
		str += "<tr><td><strong>" + tr("Store:") + "</strong></td><td>" + (bookinfo.store.isEmpty() ? tr("N/A") : bookinfo.store ) + "</td></tr>\n";
		if (!bookinfo.detail_page_URL.isEmpty()) str += "<tr><td><strong>" + tr("Details:") + "</strong></td><td><a href=\""+ bookinfo.detail_page_URL +"\" title=\""+ bookinfo.detail_page_URL +"\">Show details...</a></td></tr>\n";
		str += "</table>\n";
		str += "</body>\n</html>\n";
		ui->bookInfoTextBrowser->setHtml(str);
		book_info_shown = bookinfo;
	} else {
		ui->bookInfoTextBrowser->setText(error_msg);
	}
	ui->bookInfoTextBrowser->setEnabled(true);
}

void MainWindow::finishedBookFetch(int item_count, BookList &book_list, BookMoocherCore::BookMoocherUpdateStatus status, const QString& error_msg) {
	// Riattiva il pulsante di aggiornamento e aggiorna la statusbar
	ui->updateBookListButton->setEnabled(true);
	ui->actionUpdate->setEnabled(true);
	ui->statusBar->clearMessage();
	systray_icon->setIcon(QIcon(":/books-stack.png"));

	switch (status) {
		case BookMoocherCore::UpdateOK:
			if (systray_icon->isVisible()) {
				if (item_count > 0) {
					systray_icon->setToolTip( tr("New books to mooch: %1.\n").arg(item_count) + tr("Updated at %1").arg( QLocale::system().toString(bookmoochercore->getLastUpdateTime(), QLocale::ShortFormat) ) );
					systray_icon->showMessage(tr("New books to mooch!"), tr("There are %1 new books to mooch").arg(item_count));
				} else {
					systray_icon->setToolTip(tr("There aren't new books to mooch.\nLast updated:") + QLocale::system().toString(bookmoochercore->getLastUpdateTime(), QLocale::ShortFormat) );
				}
			}
			// bookmoochercore->loadBookList(model);
			model->setBookList(book_list);
		break;
		case BookMoocherCore::UpdateFailed:
			if (systray_icon->isVisible()) {
				systray_icon->setToolTip(tr("There was an error trying to update books data."));
				// systray_icon->showMessage(tr("Ops!"), tr("There was an error trying to update books data."), QSystemTrayIcon::Critical);
				systray_icon->showMessage(tr("Ops!"), error_msg, QSystemTrayIcon::Critical);
			} else {
				QMessageBox::critical(this, tr("Update failed!"), error_msg);
			}
			ui->statusBar->showMessage(tr("There was an error trying to update books data."));
			break;
		case BookMoocherCore::NetworkError:
			/*
			QString error_msg;
			switch(bookmoochercore->getLastNetworkReplyError()) {
				case QNetworkReply::NoError:
					error_msg = tr("No error condition.");
				break;
				case QNetworkReply::ConnectionRefusedError:
					err_msg = tr("The remote server refused the connection.");
				break;
				case QNetworkReply::RemoteHostClosedError:
					err_msg = tr("The remote server closed the connection prematurely.");
					break;
				case QNetworkReply::HostNotFoundError:
				err_msg = tr("The remote host name was not found (invalid hostname)");
				break;
				case QNetworkReply::TimeoutError:
					err_msg = tr("The connection to the remote server timed out");
				break;
				case QNetworkReply::OperationCanceledError:
					err_msg = tr("The operation was canceled.");
				break;
				case QNetworkReply::SslHandshakeFailedError:
					err_msg = tr("The SSL/TLS handshake failed.");
				break;
				case QNetworkReply::TemporaryNetworkFailureError:
					err_msg	= tr( "Temporary network failure.");
				break;
				case QNetworkReply::ProxyConnectionRefusedError:
					err_msg	= tr( "The connection to the proxy server was refused.");
				break;
				case QNetworkReply::ProxyConnectionClosedError:
					err_msg	= tr( "The proxy server closed the connection prematurely.");
				break;
				case QNetworkReply::ProxyNotFoundError:
					err_msg = tr("The proxy host name was not found (invalid proxy hostname).");
				break;
				case QNetworkReply::ProxyTimeoutError:
					err_msg	= tr( "The connection to the proxy timed out or the proxy did not reply in time to the request sent.");
				break;
				case QNetworkReply::ProxyAuthenticationRequiredError:
					err_msg	= tr( "The proxy requires authentication.");
				break;
				case QNetworkReply::ContentAccessDenied:
					err_msg	= tr("The access to the remote content was denied.");
				break;
				case QNetworkReply::ContentOperationNotPermittedError:
					err_msg = tr("The operation requested on the remote content is not permitted.");
					break;
				case QNetworkReply::ContentNotFoundError:
					err_msg = tr("The remote content was not found at the server.");
					break;
				case QNetworkReply::AuthenticationRequiredError:
					err_msg	= tr("The remote server requires authentication.");
				break;
				case QNetworkReply::ContentReSendError:
					err_msg	= tr("The request needed to be sent again, but this failed.");
				break;
				case QNetworkReply::ProtocolUnknownError:
					err_msg	= tr("Protocolo unknown.");
					break;
				case QNetworkReply::ProtocolInvalidOperationError:
					err_msg	= tr("The requested operation is invalid for this protocol.");
					break;
				case QNetworkReply::UnknownNetworkError:
					err_msg = tr("An unknown network-related error was detected.");
					break;
				case QNetworkReply::UnknownProxyError:
					err_msg = tr("An unknown proxy-related error was detected.");
					break;
				case QNetworkReply::UnknownContentError:
					err_msg = tr("An unknown error related to the remote content was detected.");
					break;
				case QNetworkReply::ProtocolFailure:
					err_msg	= tr("A breakdown in protocol was detected (parsing error, invalid or unexpected responses, etc.).");
					break;
			}*/
			if (systray_icon->isVisible()) {
				systray_icon->showMessage(tr("Ops!"), error_msg, QSystemTrayIcon::Critical);
				systray_icon->setToolTip(error_msg);
			} else {
				QMessageBox::critical(this, tr("Network error!"), error_msg);
			}
			ui->statusBar->showMessage(tr("There was a network error trying to update books data."));
			break;
	}

	// Reimposta il flag di aggiornamento in corso e abilita i menù
	enableQuit(true);
	enableButtonGroup(true);

	// Riattiva il timer
	timer->start();
}

MainWindow::~MainWindow()
{
	delete timer;
	delete ui;
	delete model;
}

void MainWindow::setupSystemTrayIcon() {
	systray_icon = new QSystemTrayIcon(QIcon(":/books-stack.png"), this);
	QMenu *systray_menu = new QMenu(this);
	// Mostra/Nascondi
	minimizeAction = systray_menu->addAction(tr("Hide BookMoocher"));
	connect(minimizeAction, SIGNAL(triggered()), this, SLOT(minimizeActionTriggered()));

	// Configura
	configureAction = systray_menu->addAction(tr("Configure..."));
	connect(configureAction, SIGNAL(triggered()), this, SLOT(on_actionConfigura_triggered()));

	// Uscita
	systray_menu->addSeparator();
	quitAction = systray_menu->addAction(tr("Quit"));
	connect(quitAction, SIGNAL(triggered()), this, SLOT(on_actionEsci_triggered()));
	systray_icon->setContextMenu(systray_menu);
}

void MainWindow::checkForNewBooks() {
	// Imposta il flag di aggiornamento in corso a true
	enableQuit(false);
	enableButtonGroup(false);

	// Disabilita il pulsante di aggiornamento forzato e aggiorna la statusbar
	ui->updateBookListButton->setEnabled(false);
	ui->actionUpdate->setEnabled(false);
	ui->statusBar->showMessage(tr("Updating..."));

	// disabilita il timer
	timer->stop();

	// Cambia l'icona sulla systray
	systray_icon->setIcon(QIcon(":/arrow-circle-double.png"));
	systray_icon->setToolTip(tr("Updating book info..."));

	// lancia la lettura
	bookmoochercore->fetchBookmoochXMLFeed();
}

void MainWindow::on_actionConfigura_triggered()
{
	// Ferma il timer
	timer->stop();

	// Mostra il dialogo per la configurazione
	ConfigDialog2 cfg(this);

	if (cfg.exec() == QDialog::Accepted) {
		// Aggiorna la configurazione
		cfg.saveSettings();

		QSettings settings;

		// Aggiorna i settaggi per il timer
		timer->setInterval(60000 * settings.value(CFG_BOOK_INFO_FETCH_INTERVAL).toInt());

		// Aggiorna i settaggi per l'azione sul doppio click nella lista dei libri
		book_list_view_action = settings.value(CFG_BOOK_LISTVIEW_DOUBLECLICK).toInt();

		// Aggiorna il modo di ricerca su anobii
		anobii_search_mode = settings.value(CFG_ANOBII_SEARCH_MODE).toInt();
	}

	// Riavvia il timer
	timer->start();
}

/**
  * Abilita o disabilita le voci di menù per l'uscita dall'applicazione
  * @param bool enable true abilita le voci, false disabilita
  **/
void MainWindow::enableQuit(bool enable) {
	ui->actionEsci->setEnabled(enable);
	quitAction->setEnabled(enable);
}


void MainWindow::on_actionInformazioni_su_Qt_triggered()
{
	QMessageBox::aboutQt(this);
}

void MainWindow::on_actionInformazioni_su_triggered()
{
	AboutDialog dlg(this);
	dlg.exec();
}

void MainWindow::on_listView_doubleClicked(QModelIndex index)
{
	switch (book_list_view_action) {
	case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_MOOCH:
		// Apre l'URL per il mooch del libro
		QDesktopServices::openUrl( QUrl( QString(CFG_BOOKMOOCH_API_MOOCH_CHOICE_URL) + QString("?asin=") + index.data().value<BookListItem>().ASIN ));
		break;
	case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SHOW_BOOK_INFO:
		ui->showBookInfoToolButton->click();
		// showSelectedBookInfo();
		break;
	case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_ADD_TO_WISHLIST:
		QDesktopServices::openUrl( QUrl( QString(CFG_BOOKMOOCH_API_WISHLIST_ADD_URL) + QString("&asin=") + index.data().value<BookListItem>().ASIN ));
		break;
	case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SAVE_FOR_LATER:
		QDesktopServices::openUrl( QUrl( QString(CFG_BOOKMOOCH_API_SAVE_FOR_LATER_ADD_URL) + QString("&asin=") + index.data().value<BookListItem>().ASIN ));
		break;
	case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SEARCH_ANOBII:
		QDesktopServices::openUrl( QUrl( QString(CFG_ANOBII_SEARCH_URL) + QString("&keyword=") + index.data().value<BookListItem>().title ));
		break;
	case CFG_BOOK_LISTVIEW_DOUBLECLICK_ACTION_SEARCH_ANOBII_BY_ISBN:
		if (!index.data().value<BookListItem>().ISBN.isEmpty() && !index.data().value<BookListItem>().ISBN.startsWith("BM", Qt::CaseInsensitive)) {
			QDesktopServices::openUrl( QUrl( QString(CFG_ANOBII_SEARCH_BY_ISBN_URL) + QString("&isbn=") + index.data().value<BookListItem>().ISBN ));
		} else {
			QMessageBox::warning(this, tr("Ops!"), tr("Book has invalid ISBN, can't search it on anobii.com."));
		}
	break;
	}
}

void MainWindow::systrayIconActivated(QSystemTrayIcon::ActivationReason reason) {
	if (reason == QSystemTrayIcon::Trigger) {
		// Esegue l'azione piuttosto che duplicare codice
		minimizeAction->trigger();
	}
}

void MainWindow::closeEvent(QCloseEvent *event) {
	if (systray_icon->isVisible()) {
		// Nasconde e ignora l'evento
		minimizeAction->setText(tr("Show BookMoocher"));
		this->hide();
		event->ignore();
	}
}

void MainWindow::minimizeActionTriggered() {
	// Nasconde o mostra la finestra
	if (this->isHidden()) {
		minimizeAction->setText(tr("Hide BookMoocher"));
		this->show();
	} else {
		minimizeAction->setText(tr("Show BookMoocher"));
		this->hide();
	}
}

void MainWindow::on_actionEsci_triggered()
{
	saveNotes();
	qApp->quit();
}

void MainWindow::moochSelectedBook() {
	QModelIndex index = ui->listView->currentIndex();
	if ( index.isValid() ) QDesktopServices::openUrl( QUrl( QString(CFG_BOOKMOOCH_API_MOOCH_CHOICE_URL) + QString("?asin=") + index.data().value<BookListItem>().ASIN ));
}

void MainWindow::showSelectedBookInfo() {
	selectStackedWidget(main_button_group->id( ui->showBookInfoToolButton ));
}

void MainWindow::wishlistAddSelectedBook() {
	QModelIndex index = ui->listView->currentIndex();
	if ( index.isValid() ) QDesktopServices::openUrl( QUrl( QString(CFG_BOOKMOOCH_API_WISHLIST_ADD_URL) + QString("&asin=") + index.data().value<BookListItem>().ASIN ));
}

void MainWindow::saveForLaterAddSelectedBook() {
	QModelIndex index = ui->listView->currentIndex();
	if ( index.isValid() ) QDesktopServices::openUrl( QUrl( QString(CFG_BOOKMOOCH_API_SAVE_FOR_LATER_ADD_URL) + QString("&asin=") + index.data().value<BookListItem>().ASIN ));
}

void MainWindow::saveNotes() {
	QFile out_file( QDesktopServices::storageLocation(QDesktopServices::DataLocation) + QDir::separator() + CFG_NOTES_FILENAME );
	// Salva in blocco
	if (out_file.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate)) {
		QTextStream out_stream( &out_file );
		out_stream << ui->notesTextEdit->toPlainText();
		out_stream.flush();
		out_file.close();
	}
}

void MainWindow::loadNotes() {
	QFile in_file( QDesktopServices::storageLocation(QDesktopServices::DataLocation) + QDir::separator() + CFG_NOTES_FILENAME );
	if (in_file.exists()) {
		// Lo carica in blocco
		if (in_file.open(QIODevice::ReadOnly | QIODevice::Text)) {
			QTextStream in_stream( &in_file );
			while (!in_stream.atEnd()) {
				ui->notesTextEdit->appendPlainText( in_stream.readLine() );
			}
			in_file.close();
		}
	}
}

void MainWindow::on_searchAnobiiToolBtn_clicked() {
	QModelIndex index = ui->listView->currentIndex();
	if ( index.isValid() ) {
		if (anobii_search_mode == CFG_ANOBII_SEARCH_BY_ISBN) {
			if (!index.data().value<BookListItem>().ISBN.isEmpty() && !index.data().value<BookListItem>().ISBN.startsWith("BM", Qt::CaseInsensitive)) {
				QDesktopServices::openUrl( QUrl( QString(CFG_ANOBII_SEARCH_BY_ISBN_URL) + QString("&isbn=") + index.data().value<BookListItem>().ISBN ));
			} else {
				QMessageBox::warning(this, tr("Ops!"), tr("Book has invalid ISBN, can't search it on anobii.com."));
			}
		} else {
			QDesktopServices::openUrl( QUrl( QString(CFG_ANOBII_SEARCH_URL) + QString("&keyword=") + index.data().value<BookListItem>().title ));
		}
	}
}
