#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QModelIndex>
#include <QSystemTrayIcon>
#include <QCloseEvent>
#include <QDateTime>
#include <QTimer>
#include <QAction>
#include "defines.h"
#include "bookmoochercore.h"
#include "booklistmodel.h"
#include "BookListProxyModel.h"

#include <QButtonGroup>

#include "BookInfoItem.h"

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

	void enableQuit(bool enable);

	void enableButtonGroup(bool enable);

public slots:
	// Aggiorna le informazioni sui libri
	void checkForNewBooks();

	// Systray: gestore degli eventi sull'icona
	void systrayIconActivated(QSystemTrayIcon::ActivationReason reason);

	// esecuzione dell'aggiornamento terminata
	void finishedBookFetch(int item_count, BookList &book_list, BookMoocherCore::BookMoocherUpdateStatus status, const QString& error_msg);

	// Esegue il mooch dell'elemento correntemente selezionato nella listview
	void moochSelectedBook();

	// usato dalla listview: mostra le info del libro correntemente selezionato
	void showSelectedBookInfo();

	// aggiunge alla wishlist
	void wishlistAddSelectedBook();

	// aggiunge alla lista "salva per dopo"
	void saveForLaterAddSelectedBook();

	// Salva le annotazioni
	void saveNotes();
	// Carica le annotazioni
	void loadNotes();

	// Mostra l'elemento selezionato nello stacked widget
	void selectStackedWidget(int id);

	void finishedBookInfoFetch(BookInfoItem &bookinfo, BookMoocherCore::BookMoocherUpdateStatus status, const QString& error_msg );

protected:
	void closeEvent(QCloseEvent *event);

private:
    Ui::MainWindow *ui;

   // Model dei dati
   BookListModel *model;

   // Proxy Model per il filtraggio
   BookListProxyModel *proxy_model;

   // Timer per l'aggiornamento
   QTimer *timer;

   // Icona nella systray
   QSystemTrayIcon *systray_icon;

   // Configura la systrayicon
   void setupSystemTrayIcon();

   // Pulsantiera con le azioni
   QButtonGroup *main_button_group;

   // ASIN del libro correntemente visualizzato
   // nel text edit con le informazioni
   QString current_shown_ASIN;

   // Flag: indica se sta aggiornando i dati sui libri
   bool is_updating;

   // Azioni
   QAction *minimizeAction,
		*maximizeAction,
		*configureAction,
		*quitAction;

   BookMoocherCore *bookmoochercore;

   // Informazioni sul libro correntemente visualizzato
   BookInfoItem book_info_shown;

   // Azione da eseguire per il doppio click sulla listview dei libri
   int book_list_view_action;

   // modalità di ricerca su anobii
   int anobii_search_mode;

 private slots:
	void on_actionEsci_triggered();
	void on_listView_doubleClicked(QModelIndex index);
	void on_actionInformazioni_su_triggered();
	void on_actionInformazioni_su_Qt_triggered();
	void on_actionConfigura_triggered();

	void on_searchAnobiiToolBtn_clicked();

	void minimizeActionTriggered();

};

#endif // MAINWINDOW_H
